// Green Gandalf's Simple Glow Shader With Lighting
//
// Created 15 December 2006, edited 24 December 2006.

matrix wvp     : WorldViewProjection ;
matrix wv      : WorldView ;
matrix mw      : World;
matrix vp      : ViewProjection;
float3 eyePos  : CameraPosition;
 
float glowWidth = 0.1;
float objectSize = 1.0;
float4 glowColour = { 0.0, 0.0, 64.0, 1.0 };
float4 ambientColour = { 0.2, 0.2, 0.2, 1.000000 };
float3 lightDirection = { 0.57735, -0.57735, 0.57735 };
float4 lightColour = { 1.0, 1.0, 1.0, 1.0 }; 

texture baseTexture < string ResourceName = "lead.jpg"; >;
sampler baseSample = sampler_state
{  Texture = <baseTexture>;
   ADDRESSU = CLAMP;
   ADDRESSV = CLAMP;
   MIPFILTER = LINEAR;
   MINFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

struct VSInput
{  float4 Pos       : POSITION;
   float3 Normal    : NORMAL;
   float4 UV        : TEXCOORD0;
};

struct VSOutput1
{  float4 Pos       : POSITION;
   float4 UV        : TEXCOORD0;
   float4 Light     : COLOR1;
};

struct VSOutput2
{  float4 Pos       : POSITION;
   float4 UV        : TEXCOORD0;
   float3 Normal    : TEXCOORD1;
   float3 View      : TEXCOORD2;
};

struct PSInput1
{  float4 UV        : TEXCOORD0;
   float4 Light     : COLOR1;
};

struct PSInput2
{  float4 UV        : TEXCOORD0;
   float3 Normal    : TEXCOORD1;
   float3 View      : TEXCOORD2;
};

struct PSOutput { float4 Col : COLOR0; };

VSOutput1 VShader1 (VSInput In, VSOutput1 Out)
{  //prepares base image and lighting
    Out.Pos = mul(In.Pos, wvp);
    float3 wNormal = normalize(mul(In.Normal, mw));
    Out.Light = saturate(dot(wNormal, -normalize(lightDirection)))
                  * lightColour + ambientColour;
    Out.UV = In.UV;
    return Out;
}

VSOutput2 VShader2 (VSInput In, VSOutput2 Out)
{  // prepares object and vectors for glow
    float4 wPos = mul(In.Pos + float4 (In.Normal * glowWidth * objectSize, 0.0), mw);
    Out.Pos = mul(wPos, vp);
    Out.Normal = normalize(mul(In.Normal, mw));
    Out.View = normalize(eyePos - wPos.xyz);
    Out.UV = In.UV;
    return Out;
}

PSOutput PShader1 (PSInput1 In, PSOutput Out)
{  Out.Col = tex2D(baseSample, In.UV) * In.Light;
   return Out;
}

PSOutput PShader2 (PSInput2 In, PSOutput Out)
{  float glow = saturate(dot(In.Normal, In.View));
   glow = glow - glow * glow;
   glow = pow(glow, 4.0);
   Out.Col = glow * glowColour;
   return Out;
}

technique glow
{  pass p1
   {  VertexShader = compile vs_2_0 VShader1();
      PixelShader  = compile ps_2_0 PShader1();
   }
   pass p2
   {  VertexShader = compile vs_2_0 VShader2();
      PixelShader  = compile ps_2_0 PShader2();
      zwriteenable = false;
      alphaBlendEnable = true;
      srcBlend = One;
      destBlend = One;
   }
}

